#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   Copyright 2013 Gary Wright wrightsolutions.co.uk/contact
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

# Tested against fabric 1.4.3
# http://ftp.uk.debian.org/debian/pool/main/f/fabric/fabric_1.4.3-1.debian.tar.gz

from __future__ import with_statement
from fabric.api import abort, cd, env, get, hide, put, run, settings, sudo
from fabric.contrib.console import confirm
from fabric.contrib.files import exists,append
from os import path
import re
from . import cmdbin
from . import package_deb as pdeb
from . import package_rpm as prpm

""" res.succeeded is preferred for newer fabric, but not res.failed
is used where backward compatibility is preferable. """

CMD_IPTABLES_SAVE = "/sbin/iptables-save "
CMD_IP6TABLES_SAVE = "/sbin/ip6tables-save "

RE_CHAIN_LABEL = re.compile(r'Chain\b')
RE_TCP_DPT22 = re.compile(r'\btcp\sdpt:22')
RE_GENERATED_BY = re.compile(r'enerated\sby\sipt')

_list_dot_d_array = []
_sources_list_object = None


def iptables_save(suffix='date',verbose_counts=False):
    save_return = False
    pretty_tuple = cmdbin.pretty_name_tuple()
    pretty_name_id = pretty_tuple[1]
    if pretty_name_id in cmdbin.DEBIAN_PLUS_DERIVATIVES2:
        requires_res = pdeb.requires_binary_package('iptables-save','iptables',assumeyes=True)
    else:
        """ package_rpm needs an implementation but code the call now anyway """
        requires_res = prpm.requires_binary_package('iptables-save','iptables',assumeyes=True)
    """ If we have iptables-save then we also have ip6tables-save so no need to repeat our check """
    total_count = 0
    generated_count = 0
    cat_lines = []
    iptables_file = '/root/iptables.error'
    if requires_res:
        from datetime import datetime as dt
        isocompact = dt.now().strftime(cmdbin.ISOCOMPACT_STRFTIME)
        assert len(isocompact) == 15
        iptables_file = "iptables.{0}".format(isocompact)
        iptables_line = "{0} > /root/{1}".format(CMD_IPTABLES_SAVE,iptables_file)
        if verbose_counts:
            ros_res = cmdbin.run_or_sudo(iptables_line)
        else:
            with hide('output'):
                ros_res = cmdbin.run_or_sudo(iptables_line)
        cat_line = "{0} /root/{1}".format(cmdbin.CMD_CAT,iptables_file)
        cat_lines = []
        if ros_res.succeeded:
            with hide('output'):
                ros_res = cmdbin.run_or_sudo(cat_line)
            cat_lines = cmdbin.run_res_lines_stripped(ros_res)
            if verbose_counts:
                for line in cat_lines:
                    """ Enhance this to have some more filtering later """
                    total_count += 1
    for line in cat_lines:
        if RE_GENERATED_BY.search(line):
            generated_count += 1
    if generated_count > 0:
        print "Generated By Count is {0}".format(generated_count)
        save_return = True
        if verbose_counts:
            print "Total Count is {0}".format(total_count)
    return save_return



